@@ -1,4 +1,5 @@
-from functools import wraps
+from functools import wraps, total_ordering
+from hypothesis.hashitanyway import HashItAnyway
 
 class SpecificationMapper(object):
     """
@@ -27,6 +28,7 @@ class SpecificationMapper(object):
         self.value_mappers = {}
         self.instance_mappers = {}
         self.__prototype = prototype
+        self.__descriptor_cache = {}
 
     def prototype(self):
       if self.__prototype: 
@@ -34,12 +36,17 @@ class SpecificationMapper(object):
       if self is self.default():
         return None
       return self.default()
+
+    def cache_specification_for_descriptor(self, descriptor, spec):
+        self.__descriptor_cache[HashItAnyway(descriptor)] = spec
     
     def define_specification_for(self, value, specification):
         self.value_mappers.setdefault(value,[]).append(specification)
+        self.__descriptor_cache = {}
 
     def define_specification_for_instances(self, cls, specification):
         self.instance_mappers.setdefault(cls,[]).append(specification)
+        self.__descriptor_cache = {}
 
     def define_specification_for_classes(self, specification,subclasses_of=None):
         if subclasses_of:
@@ -58,28 +65,56 @@ class SpecificationMapper(object):
       return self.__class__(prototype = self)
 
     def specification_for(self, descriptor):
+        k = HashItAnyway(descriptor)
+        if k in self.__descriptor_cache:
+            return self.__descriptor_cache[k]
+        
         for h in self.__find_specification_handlers_for(descriptor):
             try:
-                return h(self, descriptor)
+                r = h(self, descriptor)
+                break
             except NextInChain:
                 pass
-        return self.missing_specification(descriptor)
+        else: 
+            r = self.missing_specification(descriptor)
+
+        self.__descriptor_cache[k] = r
+        return r
 
     def __find_specification_handlers_for(self, descriptor):
         if safe_in(descriptor, self.value_mappers):
             for h in reversed(self.value_mappers[descriptor]):
                 yield h 
         tk = typekey(descriptor)
-        if tk in self.instance_mappers:
-            for h in reversed(self.instance_mappers[tk]):
-                yield h
+        for h in self.__instance_handlers(tk):
+            yield h
         if self.prototype():
             for h in self.prototype().__find_specification_handlers_for(descriptor):
                 yield h
 
+    def __instance_handlers(self, tk):
+        for c, hs in sorted(self.instance_mappers.items(), key = lambda x: ClassSorter(x[0])):
+            if issubclass(tk, c):
+                for h in reversed(hs):
+                    yield h
+
     def missing_specification(self, descriptor):
         raise MissingSpecification(descriptor)
 
+@total_ordering
+class ClassSorter(object):
+    def __init__(self, cls):
+        self.cls = cls
+
+    def __eq__(self, that):
+        return self.cls == that.cls
+
+    def __lt__(self, that):
+        if self.cls == that.cls: return False
+        elif issubclass(self.cls, that.cls): return True
+        elif issubclass(that.cls, self.cls): return False
+        else: return self.cls.__name__ < that.cls.__name__ 
+
 def typekey(x):
     try:
         return x.__class__
